@@ -21,6 +21,9 @@ struct gvm_configuration {
 	gboolean autoplay_dvd;
 	char *autoplay_dvd_command;
 	
+	gboolean autoplay_vcd;
+	char *autoplay_vcd_command;
+	
 	gboolean autophoto;
 	char *autophoto_command;
 	
