@@ -77,6 +77,8 @@ enum {
 	AUTOPLAY_CDA_COMMAND,
 	AUTOPLAY_DVD,
 	AUTOPLAY_DVD_COMMAND,
+	AUTOPLAY_VCD,
+	AUTOPLAY_VCD_COMMAND,
 	AUTOPRINTER,
 	AUTOPRINTER_COMMAND,
 	AUTORUN,
@@ -103,6 +105,8 @@ static struct {
 	{ GCONF_ROOT "autoplay_cda_command",      TYPE_STRING, &config.autoplay_cda_command      },
 	{ GCONF_ROOT "autoplay_dvd",              TYPE_BOOL,   &config.autoplay_dvd              },
 	{ GCONF_ROOT "autoplay_dvd_command",      TYPE_STRING, &config.autoplay_dvd_command      },
+	{ GCONF_ROOT "autoplay_vcd",              TYPE_BOOL,   &config.autoplay_vcd              },
+	{ GCONF_ROOT "autoplay_vcd_command",      TYPE_STRING, &config.autoplay_vcd_command      },
 	{ GCONF_ROOT "autoprinter",               TYPE_BOOL,   &config.autoprinter               },
 	{ GCONF_ROOT "autoprinter_command",       TYPE_STRING, &config.autoprinter_command       },
 	{ GCONF_ROOT "autorun",                   TYPE_BOOL,   &config.autorun                   },
@@ -653,6 +657,33 @@ gvm_ask_autorun (const char *path)
 	return autorun;
 }
 
+static gboolean
+gvm_check_dir (const char *dirname, const char *name)
+{
+	gboolean exists = FALSE;
+	struct dirent *dent;
+	struct stat st;
+	char *path;
+	DIR *dir;
+	
+	if (!(dir = opendir (dirname)))
+		return FALSE;
+	
+	while ((dent = readdir (dir))) {
+		if (!g_ascii_strcasecmp (dent->d_name, name)) {
+			path = g_build_filename (dirname, dent->d_name);
+			if (stat (path, &st) == 0 && S_ISDIR (st.st_mode))
+				exists = TRUE;
+			g_free (path);
+			break;
+		}
+	}
+	
+	closedir (dir);
+	
+	return exists;
+}
+
 /*
  * gvm_check_dvd - is this a Video DVD?  If so, do something about it.
  *
@@ -661,25 +692,32 @@ gvm_ask_autorun (const char *path)
 static gboolean
 gvm_check_dvd (const char *udi, const char *device, const char *mount_point)
 {
-	char *path;
-	gboolean retval;
+	gboolean is_dvd;
 	
-	path = g_build_path (G_DIR_SEPARATOR_S, mount_point, "video_ts", NULL);
-	retval = g_file_test (path, G_FILE_TEST_IS_DIR);
-	g_free (path);
+	is_dvd = gvm_check_dir (mount_point, "video_ts");
 	
-	/* try the other name, if needed */
-	if (retval == FALSE) {
-		path = g_build_path (G_DIR_SEPARATOR_S, mount_point,
-				     "VIDEO_TS", NULL);
-		retval = g_file_test (path, G_FILE_TEST_IS_DIR);
-		g_free (path);
-	}
-	
-	if (retval && config.autoplay_dvd)
+	if (is_dvd && config.autoplay_dvd)
 		gvm_run_command (config.autoplay_dvd_command, udi, device, mount_point);
 	
-	return retval;
+	return is_dvd;
+}
+
+/*
+ * gvm_check_vcd - is this a Video CD?  If so, do something about it.
+ *
+ * Returns TRUE if this was a Video CD and FALSE otherwise.
+ */
+static gboolean
+gvm_check_vcd (const char *udi, const char *device, const char *mount_point)
+{
+	gboolean is_vcd;
+	
+	is_vcd = gvm_check_dir (mount_point, "vcd");
+	
+	if (is_vcd && config.autoplay_dvd)
+		gvm_run_command (config.autoplay_vcd_command, udi, device, mount_point);
+	
+	return is_vcd;
 }
 
 /*
@@ -805,41 +843,21 @@ gvm_run_camera (const char *udi, const char *device, const char *mount_point)
 static gboolean
 gvm_check_photos (const char *udi, const char *device, const char *mount_point)
 {
-	int have_dcim = FALSE;
-	struct dirent *dent;
-	DIR *dir;
+	DBusError error;
 	
-	if (!(dir = opendir (mount_point)))
+	if (!gvm_check_dir (mount_point, "dcim"))
 		return FALSE;
 	
-	while ((dent = readdir (dir))) {
-		if (!g_ascii_strcasecmp (dent->d_name, "dcim")) {
-			DBusError error;
-			char *dcim_path;
-			struct stat st;
-			
-			dcim_path = g_build_path (G_DIR_SEPARATOR_S, mount_point, dent->d_name, NULL);
-			if (stat (dcim_path, &st) == 0 && S_ISDIR (st.st_mode)) {
-				dbg ("Photos detected: %s\n", dcim_path);
-				
-				/* add the "content.photos" capability to this device */
-				dbus_error_init (&error);
-				if (!libhal_device_add_capability (hal_ctx, udi, "content.photos", &error)) {
-					warn ("failed to set content.photos on %s: %s", device, error.message);
-					dbus_error_free (&error);
-				}
-				
-				have_dcim = TRUE;
-			}
-			g_free (dcim_path);
-			
-			break;
-		}
+	dbg ("Photos detected: %s\n", mount_point);
+	
+	/* add the "content.photos" capability to this device */
+	dbus_error_init (&error);
+	if (!libhal_device_add_capability (hal_ctx, udi, "content.photos", &error)) {
+		warn ("failed to set content.photos on %s: %s", device, error.message);
+		dbus_error_free (&error);
 	}
 	
-	closedir (dir);
-	
-	return have_dcim;
+	return TRUE;
 }
 
 /*
@@ -889,6 +907,9 @@ gvm_autorun (const char *udi, const char *device, const char *mount_point)
 	if (gvm_check_dvd (udi, device, mount_point))
 		return;
 	
+	if (gvm_check_vcd (udi, device, mount_point))
+		return;
+	
 	if (config.autophoto && gvm_check_photos (udi, device, mount_point)) {
 		if (gvm_prompt (GVM_PROMPT_IMPORT_PHOTOS, 0, NULL) == GVM_RESPONSE_IMPORT_PHOTOS) {
 			gvm_run_camera (udi, device, mount_point);
@@ -909,8 +930,10 @@ gvm_autorun (const char *udi, const char *device, const char *mount_point)
 			argv[0] = path;
 			argv[1] = NULL;
 			
-			if (access (path, X_OK))
+			if (access (path, X_OK)) {
+				g_free (path);
 				continue;
+			}
 			
 			if (gvm_ask_autorun (path)) {
 				GError *error = NULL;
